require File.expand_path(File.dirname(__FILE__) + "/../spec_helper")
require 'more_accurate/more_accurate'

describe MoreAccurate::MoreAccurate do
  before(:each) do
    @initial_word_list =  ['cat','cac','cab','hat','one','four','ax','fives']
    @word = '___'
    @player = MoreAccurate::MoreAccurate.new
    @player.word_list = @initial_word_list
    @player.new_game(6)
  end
  
  it "should be instantiable with no paramters" do
    lambda { MoreAccurate::MoreAccurate.new }.should_not raise_error
  end

  it "should return the initial word list before any guesses" do
    @player.word_list.should == @initial_word_list
  end

  it "should respond to guess with a one_character String to try" do
    @player.guess(@word,nil).class.should == String
    @player.guess(@word,nil).size.should == 1
  end

  it "a guess should filter initial word list by the size of the word pattern" do
    @player.guess(@word,nil)
    @player.word_list.should == ['cat','cac','cab','hat','one']
  end

  it "filters initial word list by a word pattern" do
    @word = '_a_'
    @player.guess(@word,nil)
    @player.word_list.should == ['cat','cac','cab','hat']
  end

  it "filters word list by a word pattern AND no other spots for last correct letter" do
    @word = '___'
    @player.guess(@word,nil)
    @player.correct_guess(nil)
    @player.regex_from_word_pattern(@word).should == /^[^\d][^\d][^\d]$/i
    @player.word_list.should == ["cat", "cac", "cab", "hat", "one"]
    @word = 'c__'
    @player.guess(@word,nil)
    @player.correct_guess('c')
    @player.regex_from_word_pattern(@word).should == /^c[^c][^c]$/i
    @player.guess(@word,nil)
    @player.word_list.should == ['cat','cab']
  end

  it "should list letters left in word list that are not already known to be correct" do
    @word = '_a_'
    @player.guess(@word,nil)
    @player.correct_guess('a')
    @player.potential_remaining_letters.sort.should == %w| b c t h |.sort
  end

  it "should list letters left in word list that are not already known to be incorrect" do
    @player.guess(@word,nil)
    @player.incorrect_guess('h')
    @player.potential_remaining_letters.sort.should == %w| c a t b o n e |.sort
  end

  it "should track correct and incorrect already_tried_letters" do
    @player.already_tried_letters.should == []
    @player.correct_guess('a')
    @player.already_tried_letters.should == ['a']
    @player.incorrect_guess('h')
    @player.already_tried_letters.should == ['a','h']
  end

  it "should remove words from word list that match incorrect guesses" do
    @player.guess(@word,nil)
    @player.incorrect_guess('c')
    @player.word_list.should == ['hat','one']
  end

  it "count partition size that don't match a letter" do
    @player.word_list = ['cat','cac','cab','hat','one'] ; @player.new_game(nil)
    @player.filter_word_list_by_incorrect_letter('o').size.should == 4
    @player.word_list = ['cat','cac','cab','hat','one'] ; @player.new_game(nil)
    @player.filter_word_list_by_incorrect_letter('c').size.should == 2
    @player.word_list = ['cat','cac','cab','hat','one'] ; @player.new_game(nil)
    @player.filter_word_list_by_incorrect_letter('a').size.should == 1
    @player.word_list = ['cat','cac','cab','hat','one'] ; @player.new_game(nil)
    @player.filter_word_list_by_incorrect_letter('t').size.should == 3
    @player.word_list = ['cat','cac','cab','hat','one'] ; @player.new_game(nil)
    @player.filter_word_list_by_incorrect_letter('b').size.should == 4
    @player.word_list = ['cat','cac','cab','hat','one'] ; @player.new_game(nil)
    @player.filter_word_list_by_incorrect_letter('z').should == nil
  end

  it "should respond to guess with the letter that has smallest large partition" do
    @player.guess(@word,nil).should == 'c'
  end

  it "should not break when playing more than one game" do
    @initial_word_list =  ['cat','cac','cab','hat','one','four','a2','fives','afbc','xyfz']
    @word = '___'
    @player = MoreAccurate::MoreAccurate.new
    @player.new_game(nil)
    @player.word_list = @initial_word_list
    @player.new_game(nil)
    @player.guess(@word,nil).should == 'c'
    @word = '____'
    @player.new_game(nil)
    @player.guess(@word,nil)
    @player.word_list.should == ['four','afbc','xyfz']
    @player.new_game(nil)
    @player.guess(@word,nil).should == 'f'
  end
  
end


